﻿// SetSpaceKerning.jsx

/*
    
Всем пробелам в выделенном тексте устанавливается определённый кернинг, и трекинг, равный нулю.
Обрабатываются пробелы, оформленные выбранным абзацным стилем. 
Очевидно, что для обработки сносок надо выделить основной текст и указать нужный стиль. Выделять текст в сносках не надо.
Поиск среди символов, окрашенных чёрным цветом.
Обрабатываются только пробелы, шпации пропускаются,

Изменение кернинга - это способ сделать пробелы шире. В компьютерном наборе они стандартно примерно в четверть круглой.
В горячем наборе их ширина варьировалась от трети круглой до половины.
И если есть задача сделать книгу, чтобы её оформление в плане ширины пробелов и более свободного расстояния между буквами
было похоже на ручной набор, то этот скрипт поможет в решении такой задачи. Расстояние между буквами можно увеличить
с помощью обычного трекинга, но трекинг влияет и на ширину пробелов. 
А данный скрипт, делая пробелы достаточно широкими, исключает влияние изменения трекинга на них, он всегда нулевой.
В результате можно независимо регулировать и межбуквенный трекинг, и ширину пробелов.
Пробелы не будут все как один одинаковой ширины, но если стандартно их ширина варьируется около 0,25% круглой, например, в диапазоне 0,19 —0,31%,
то с этой программой это ширина может варьироваться примерно так, 0,37—0,58%. И это уже очень похоже на то, что было в горячем наборе.

Если результат работы программы не совпал с ожиданиями, клавиши Ctrl+Z откатят вёрстку к состянию до запуска 
15.10.2024
*/


#targetengine "SetSpaceKerning"
#include "Messages.jsxinc"

app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll; // см. http://adobeindesign.ru/2008/10/24/restore-ui/

var ProgressBar = function(title) { // ProgressBar
var w = new Window('palette', title, {x:0, y:0, width:940, height:30},{closeButton: true});
pb = w.add('progressbar', {x:20, y:8, width:900, height:12});
w.center();
this.reset = function(maxValue) {
    pb.value = 0;
    pb.maxvalue = maxValue;
    w.show();
    w.update();
    }
this.hit = function() {++pb.value; w.update();}
this.hide = function() {w.hide();}
this.close = function() {w.close(); }
} // ProgressBar
////////////////////////////

var programTitul = mes030; // "Изменение кернинга пробелов"
////////
if(app.documents.length == 0) {
alert(mes003, programTitul);	 // "Нет открытого документа."
exit();    
}
var kernMin = -100;
var kernMax = 1000;
////
var doc = app.activeDocument;
doc.textPreferences.smartTextReflow = true;
var docName = decodeURI(doc.name);
var paraStyles = [];    
var paraStyleID = [];  
while (paraStyles.length > 0) paraStyles.shift();
while (paraStyleID.length > 0) paraStyleID.shift();
var myParagraphStyles = doc.allParagraphStyles;
var myParagraphStyleName, obj;
for (i = 0; i < myParagraphStyles.length ; i++) { // i++
    myParagraphStyleName = myParagraphStyles[i].name;
    obj = myParagraphStyles[i];
    while(obj.parent instanceof ParagraphStyleGroup) {
        myParagraphStyleName = obj.parent.name + ":" + myParagraphStyleName;
        obj = obj.parent;        
        }
    paraStyles.push(myParagraphStyleName);
    paraStyleID.push(myParagraphStyles[i].id);
    } // i++
paraStyles.shift(); // удаление стиля No Paragraph Style
paraStyleID.shift();
var horUnit = doc.viewPreferences.horizontalMeasurementUnits;
doc.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.POINTS;
/////
var styleName = "";
var stylePosition = 0;

var myDefSetName  = "#kern.ini"; // это файл установок программы
var myScriptFile = myGetScriptPath();
var myScriptFolder = decodeURI(myScriptFile.path);
var setFolderPath = new Folder (myScriptFolder + "/" + "sets"); 
if (setFolderPath.exists == false) {   
    setFolderPath.create();
    }
var myFilePath = decodeURI(myScriptFolder + "/sets/" + myDefSetName); 
var myDataFile = new File (myFilePath);
if (myDataFile.open("r")) { // такой файл есть 
styleName = myDataFile.readln();
stylePosition = myDataFile.readln();
if (paraStyles[stylePosition] != styleName) stylePosition = 0;
myDataFile.close();
} // "r"

var win = new Window ("palette", programTitul);
win.alignChildren = ["fill", "fill"];
var allSpaces = win.add("checkbox", undefined, mes032); // "Все пробелы, независимо от абзацного стиля"
allSpaces.value = false;
var styleGroup = win.add("group");
styleGroup.orientation = "row";
styleGroup.enabled = true;
var style = styleGroup.add("dropdownlist",undefined, paraStyles);
var styleListLength = 110;
style.minimumSize.width = style.maximumSize.width = styleListLength;
style.selection = stylePosition;
styleGroup.add("statictext", undefined, mes033); // "Используемый абзацный стиль"
var kernGroup = win.add("group");
kernGroup.orientation = "row";
var kernValue = kernGroup.add("edittext {justify: 'center'}");
kernValue.characters = 10;
kernValue.text = "?";
var infoKernText = mes034 + kernMin + " … " + kernMax + ")"; // "Величина кернинга ("
var kernLine = kernGroup.add("statictext", undefined, infoKernText);
separator1 = win.add ("panel"); 
separator1.minimumSize.height = separator1.maximumSize.height = 1;
separator1.alignment = ["fill", "fill"];
var buttons = win.add("group");
buttons.orientation = "row";
buttons.alignChildren = ["center", "center"];
var info = buttons.add("button", [0,0,28,28], "?");
var actionText1 = mes013; // "Выполнить"
var actionText2 = mes014; // "Обработка"
var action = buttons.add("button", [0,0,270,28], actionText1);
action.enabled = false;
///
allSpaces.onClick = function () { // allSpaces
if (allSpaces.value == true) styleGroup.enabled = false; else styleGroup.enabled = true;
} // allSpaces
///
info.onClick = function() { // info.onClick 
var infoText = mes037; // "Всем пробелам в выделенном тексте устанавливается определённый кернинг, при этом их трекинг делается равным нулю.\nЗначение кернинга задаётся целым числом.\n\nОбрабатываются пробелы, оформленные выбранным абзацным стилем. Обрабатываются только пробелы, шпации пропускаются.\nПоиск среди символов шириной 100%, окрашенных чёрным цветом.\n\nДля обработки сносок надо выделить основной текст и указать нужный стиль. Выделять текст в сносках не надо.\n\nИзменение кернинга — это способ сделать пробелы шире. В компьютерном наборе они стандартно примерно в четверть круглой.\nВ горячем наборе их ширина варьировалась от трети круглой до половины, иногда чуть больше. И межбуквенное расстояние было несколько больше. На страницах тех книг больше воздуха, они более комфортны для чтения.\n\nИ если есть желание сделать книгу, чтобы её оформление в плане ширины пробелов и более свободного расстояния между буквами было похоже на ручной набор, то этот скрипт поможет в решении такой задачи. Расстояние между буквами можно увеличить с помощью обычного трекинга, но трекинг влияет и на ширину пробелов. \nА данный скрипт, позволяющий делать пробелы любой ширины, исключает влияние изменения трекинга на них, он всегда нулевой.\nВ результате можно независимо регулировать и межбуквенный трекинг, и ширину пробелов.\n\nПробелы не будут все как один одинаковой ширины, если стандартно сейчас их ширина близка к 0,25% круглой, плюс минус конечно, например, в диапазоне 0,19 – 0,31%, то с этой программой ширина может варьироваться примерно так, 0,37 – 0,58%.\nИ это уже похоже на то, что было в горячем наборе.\n\nПосле завершения обработки выделенного текста готовится информация о наименьшем и наибольшем пробеле, сначала размер в пунктах, затем процентное отношение этого размера к кеглю. Данная информация сохраняется во всплывающей подсказке к кнопке 'Выполнить'.\n\nЕсли результаты работы программы не совпали с ожиданиями, то клавиши Ctrl+Z вернут вёрстку к состоянию, в каком она была перед началом обработки.\n\nhttps://dotextok.ru  |  dotextok@gmail.com"
//alert(infoText, programTitul);
var wn = new Window ("dialog", programTitul);
var data = wn.add("edittext", [0, 0, 850, 450], infoText, {multiline: true});
wn.layout.layout();
wn.show();
return;
} // info.onClick
/////
kernValue.onChanging = function () { // kernValue.onChanging
var redColor = [Number(1), Number(0), Number(0)];    
var blackColor = [Number(0), Number(0), Number(0)];
var valid = /^[-\d]+$/.test (kernValue.text) && !isNaN(kernValue.text); 
if (Number(kernValue.text) < kernMin || Number(kernValue.text) > kernMax) valid = false;
if (valid == false) {
    kernLine.graphics.foregroundColor = kernLine.graphics.newPen (win.graphics.PenType.SOLID_COLOR, redColor, 1);
    action.enabled = false;
    }
else {
    kernLine.graphics.foregroundColor = kernLine.graphics.newPen (win.graphics.PenType.SOLID_COLOR, blackColor, 1);
    action.enabled = true;    
    }
} // kernValue.onChanging
///
action.onClick = function () { // action.onClick
if (decodeURI(app.activeDocument.name) != docName) {
   alert(mes016 + docName + mes017, programTitul); // "Это не тот документ, что был на экране в момент запуска данного скрипта. Тогда была работа с файлом '" // "', а сейчас открыта другая работа."
   return;
    }
app.doScript(myTextProcessingAction, ScriptLanguage.JAVASCRIPT, [], UndoModes.ENTIRE_SCRIPT, 'myTextProcessingAction');
return;
////
function myTextProcessingAction() { // myTextProcessingAction
var sel = app.selection[0];
if (sel == null || (sel.constructor.name != "Paragraph" && sel.constructor.name != "Text" && sel.constructor.name != "TextColumn" && sel.constructor.name != "TextStyleRange"  && sel.constructor.name != "Line")) {
    alert(mes040,programTitul); // "Перед запуском скрипта должен быть выделен текст."
    return;    
    }
try {
    var pTest = sel.characters[-1].parentTextFrames[0].parentPage.isValid == true;
    }
catch(e) {
    alert(mes021, programTitul); // "Похоже, тут есть вытесненный текст."
    return;
    }
if (sel.parent.constructor.name == "Footnote") {
    alert(mes042,programTitul); // "Надо выделить основной текст, а не сноску."
    return;    
    }
action.text = actionText2;
win.update();
var space_letter_IP = [];
var story= sel.parent;
var pBar = new ProgressBar(mes043);  // "1/4 Поиск пробелов в выделенном тексте / прогрессбара не будет /"
pBar.reset(0);
app.findGrepPreferences = NothingEnum.nothing;
app.changeGrepPreferences = NothingEnum.nothing;
app.findChangeGrepOptions.includeFootnotes = true;
while (space_letter_IP.length > 0) space_letter_IP.pop();
app.findGrepPreferences.findWhat = " "; // ищем именно пробелы, шпации пропускаем.
var usedStyle = app.activeDocument.paragraphStyles.itemByID(paraStyleID[style.selection.index]);
if (allSpaces.value == false) app.findGrepPreferences.appliedParagraphStyle = usedStyle;
app.findGrepPreferences.horizontalScale = 100;
app.findGrepPreferences.fillColor = "Black";
var spatios = sel.findGrep();   
if (spatios.length == 0) {
    pBar.close();
    alert(mes044,programTitul); // "Ничего не найдено."
    action.text = actionText1;
    win.update();
    return;        
    }   
/////
var allSpacesLengthS = 0;
var spaceNumS = 0;
var pBar = new ProgressBar(mes045);  // "2/4 До изменения кернинга: вычисление средней ширины пробела"
var linesNum = sel.lines.length;
pBar.reset(linesNum);
var spMin = 99999;
var spMax = 0;
var spLength;
var string;
for (var s = linesNum-1; s >= 0 ; s--, pBar.hit()) { // s--   
    string = sel.lines[s];
    if (string.length < 4) continue;
    app.findGrepPreferences = NothingEnum.nothing;
    app.changeGrepPreferences = NothingEnum.nothing;
    app.findChangeGrepOptions.includeFootnotes = true;    
    var usedStyle = app.activeDocument.paragraphStyles.itemByID(paraStyleID[style.selection.index]);
    if (allSpaces.value == false) app.findGrepPreferences.appliedParagraphStyle = usedStyle;
    app.findGrepPreferences.fillColor = "Black";    
    app.findGrepPreferences.findWhat = " ";
    app.findGrepPreferences.horizontalScale = 100;
    var spatios_tmp = string.findGrep();
    for (var i = 0; i < spatios_tmp.length-1; i++) { // i++
        // крайний пробел в строке не учитывается, т.к. он может быть последним пробелом, и в этом случае компоновщик может сделать его ширину близкой к нулю.
        spLength = Math.abs(spatios_tmp[i].endHorizontalOffset - spatios_tmp[i].horizontalOffset);
        allSpacesLengthS = allSpacesLengthS + spLength;
        spaceNumS = spaceNumS+1;
        } // i++    
    } // s--
// К этому моменту в переменной allSpacesLengthS собрана сумма ширин всех пробелов, и известно их число, это переменная spaceNumS.
////
var pBar = new ProgressBar("3/4 " + programTitul); 
pBar.reset(spatios.length);
for (i = spatios.length-1; i >= 0 ; i--, pBar.hit()) { // i-- 
    spatios[i].tracking = 0;
    var newIP = spatios[i].index+1; // это позиция точки вставки между пробелом и словом
    spatios[i].parent.insertionPoints[newIP].kerningValue = Number(kernValue.text);    
    }  // i-- 
pBar.close();
try {
    var pTest = sel.characters[-1].parentTextFrames[0].parentPage.isValid == true;
    }
catch(e) {
    alert(mes046, programTitul); // "Похоже, в результате изменения кернинга появился вытесненный текст. По этой причине не удалось выяснить разброс размеров пробелов. Если это нужно, то добавьте страницы и запустите программу снова. Она перезапишет эти значения кернинга, но поскольку вытесненного текста не будет, то выполнится поиск наименьшего и наибольшего пробелов."
    action.text = actionText1;
    win.update();
    return;
    }
var allSpacesLengthF = 0;
var spaceNumF = 0;
var pBar = new ProgressBar(mes047);  // "4/4 После обработки: поиск самого большого и самого маленького пробела, и вычисление средней ширины пробела"
linesNum = sel.lines.length;
pBar.reset(linesNum);
var spMin = 99999;
var spMax = 0;
var spLength;
var string;
for (var s = linesNum-1; s >= 0 ; s--, pBar.hit()) { // s--   
    string = sel.lines[s];
    if (string.length < 4) continue;
    app.findGrepPreferences = NothingEnum.nothing;
    app.changeGrepPreferences = NothingEnum.nothing;
    app.findChangeGrepOptions.includeFootnotes = true;    
    var usedStyle = app.activeDocument.paragraphStyles.itemByID(paraStyleID[style.selection.index]);
    if (allSpaces.value == false) app.findGrepPreferences.appliedParagraphStyle = usedStyle;
    app.findGrepPreferences.fillColor = "Black";    
    app.findGrepPreferences.findWhat = " ";
    app.findGrepPreferences.horizontalScale = 100;
    var spatios = string.findGrep();
    for (var i = 0; i < spatios.length-1; i++) { // i++
        // крайний пробел в строке не учитывается, т.к. он может быть последним пробелом, и в этом случае компоновщик может сделать его ширину близкой к нулю.
        spLength = Math.abs(spatios[i].endHorizontalOffset - spatios[i].horizontalOffset);
        allSpacesLengthF = allSpacesLengthF + spLength;
        spaceNumF = spaceNumF+1;

        if (spLength > spMax) { spMax = spLength; }
        if (spLength < spMin) { spMin = spLength; } 
        } // i++    
    } // s--
pBar.close();
if (allSpaces.value == false) { // ! allSpaces.value
    var pntSz = usedStyle.pointSize;
    var minFraction = ((spMin/pntSz)*100).toFixed(0);
    var maxFraction = ((spMax/pntSz)*100).toFixed(0);
    var rezMin = spMin.toFixed(2) + " / " + minFraction + "%";
    var rezMax = spMax.toFixed(2) + " / " + maxFraction + "%";
    var avarageSpaceLengthF = (allSpacesLengthF/spaceNumF).toFixed(2);
    var averageSpaceFractionF = (((allSpacesLengthF/spaceNumF)/pntSz)*100).toFixed(0) + "%"; 
    var avarageSpaceLengthS = (allSpacesLengthS/spaceNumS).toFixed(2);
    var averageSpaceFractionS = (((allSpacesLengthS/spaceNumS)/pntSz)*100).toFixed(0) + "%"; 
    var rezLine = mes048 + paraStyles[style.selection.index] + mes049 + kernValue.text + mes050 + rezMin + mes051 + rezMax + mes052 + avarageSpaceLengthF + " | " + averageSpaceFractionF + mes053 + avarageSpaceLengthS + " | " + averageSpaceFractionS; // "Стиль: " // "\nКернинг = " // "\rРазброс ширины пробелов ( пт / доля значения круглой ):\rМин: " // "  |  Макс.: " // "\rСредняя ширина пробела ( пт / доля значения круглой ): " // "\rСредняя ширина пробела до изменения кернинга: "
    action.helpTip = rezLine;
   // alert(rezLine + mes054, programTitul); // "\r\rЭта информация сохранена в всплывающей подсказке к кнопке 'Выполнить'."
    }  // ! allSpaces.value
else {
    showMes(mes055); // "В выделенной области изменён кернинг всех пробелов."
    }
action.text = actionText1;
win.update();
return;
}  // myTextProcessingAction
} // action.onClick
////
win.onClose = function() { 
doc.viewPreferences.horizontalMeasurementUnits = horUnit;
myDataFile.open("w");
myDataFile.writeln(style.selection);
myDataFile.writeln(Number(style.selection));
myDataFile.close();
app.findGrepPreferences = app.changeGrepPreferences = null;  
app.activate();
}
/////////////////////
win.show();
////
function myGetScriptPath() { // myGetScriptPath
try { return app.activeScript; }
catch(e) { return File(e.fileName); }
} // myGetScriptPath()
////
function showMes (message) { // showMes
var ewin = new Window ("palette", message, {x:0, y:0, width:500, height:12}); 
ewin.center();
ewin.show();
ewin.update();   
$.sleep(1500);
ewin.close();
return;
} // showMes